//+------------------------------------------------------------------------------------------------------------+
//| GBP/USD M1 Breakout Trading System Implementation                                                          |
//| Original Concept & Code: http://www.strategybuilderfx.com/showpost.php?p=166491&postcount=5807             |
//| Modified By Ruby Salton, www.tornadofx.co.il                                                               |
//| Copyright  2006, Ruby Salton                                                                              |
//+------------------------------------------------------------------------------------------------------------+
#property copyright "Copyright  2006, Ruby Salton"
#property link      "www.tornadofx.co.il"

//+------------------------------------------------------------------------------------------------------------+
//| Version 0.01: 1. Basic implementation                                                                      |
//+------------------------------------------------------------------------------------------------------------+
//| Version 0.02: 1. Bug fix - placing stop orders when trigger price is too close to the market price         |
//+------------------------------------------------------------------------------------------------------------+
//| Version 0.03: 1. Bug fix - placing SL on reverse band                                                      |
//+------------------------------------------------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| Constants                                                        |
//+------------------------------------------------------------------+ 
#define EA_NAME                     "TFX Breakouts v0.01"
#define MAGIC_NUMBER                88833
#define SLIPPAGE                    3
#define ATR_TRAILING_TIME_FRAME     PERIOD_H1
#define ATR_TRAILING_PERIOD         4
#define ATR_TRAILING_BAR_SHIFT      1
#define TOP_RANGE_BAND              0
#define BOTTOM_RANGE_BAND           1

//+------------------------------------------------------------------+
//| External parameters                                              |
//+------------------------------------------------------------------+ 

extern string  Header.1                         = "-------------------------------   Money Management Parameters   -------------------------------";
extern double  Normal.Order.Size                = 1;        //When set to 0, autumatic money management kicks in
extern double  Risk.Factor                      = 0.2;      //How much free margin to risk

extern string  Header.2                         = "-------------------------------   Order Management Parameters  --------------------------------";
extern int     Trade.Open.Time                  = 11;       //Time to place orders
extern int     DST.Shift                        = -1;       //DST diff between local time and server time
extern int     Range.Priod                      = 2;        //Range period in hours before placing the orders
extern int     Max.Range.Pips                   = 999;       //Skip new trades for ranges wider than this value
extern int     Safty.Channel.Pips               = 10;       //Safty channel (outside of the range) for placing orders
extern int     Pending.Trades.Expiration        = 10;       //For how long do we hold our pending non activated orders (in Hours)
extern int     Max.SL.Pips                      = 80;       //Maximum pips to risk
extern int     TP.Pips                          = 999;      //TP in pips. Set to large value to exit trades by SL only (trailing stop)

extern string  Header.3                         = "---------------------------------   Profit Lock Parameters  -----------------------------------";
extern int     Lock.Profit.Trigger.Pips         = 20;       //When profit reaches this value we would like to lock some profit. Set to 0 to ignore profit lock
extern int     Pips.To.Lock                     = 5;        //How many pips to lock when reaching Lock.Profit.Trigger.Pips

extern string  Header.4                         = "---------------------------------   Trailing Stop Parameters  ---------------------------------";
extern int     Trailing.Stop.Trigger.Pips       = 25;       //When profit reaches this value we would like to begin trailing stop. Set to 0 to ignore trailing stop 
extern int     ATR.Trailing.SL.Factor           = 4;        //How many ATRs the stop will be placed
extern int     Max.Trailing.Pips                = 999;      //Max distance of in pips between the new profit record and the SL. Set large value to ignore


//+------------------------------------------------------------------+
//| Variables                                                        |
//+------------------------------------------------------------------+ 
string pair.Symbol;
double pair.Point.Value;
double pair.Min.Lot;
double pair.Max.Lot;
double pair.Lot.Step;
double pair.Stop.Level;
double pair.One.Pip.Value;
int    chart.Period;
double normal.Spread.Value;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+ 
int init()
{
   pair.Symbol = Symbol();
   pair.Min.Lot = MarketInfo(pair.Symbol, MODE_MINLOT);
   pair.One.Pip.Value = MarketInfo(pair.Symbol, MODE_TICKVALUE);
   pair.Max.Lot = MarketInfo(pair.Symbol, MODE_MAXLOT);
   pair.Lot.Step = MarketInfo(pair.Symbol, MODE_LOTSTEP);
   pair.Point.Value = MarketInfo(pair.Symbol, MODE_POINT);
   pair.Stop.Level = MarketInfo(pair.Symbol, MODE_STOPLEVEL);
   normal.Spread.Value = MarketInfo(pair.Symbol, MODE_SPREAD) * pair.Point.Value;
   chart.Period = Period();
   DrawLabels();
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+ 
int start()
{
   HandleProfitLock();
   HandleTrailingStop();
   
   if (CanSetNewOrders())
   {
      double dailyRange[2];
      DetermineDailyRange(dailyRange);
      HandleNewOrders(dailyRange);
   }
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+ 
bool CanSetNewOrders()
{
   datetime ordersTime = StrToTime((Trade.Open.Time + DST.Shift) + ":00");

   if (OrdersFromCurrentSessionExists(ordersTime, MODE_TRADES, OrdersTotal()))
      return (false);
      
   if (OrdersFromCurrentSessionExists(ordersTime, MODE_HISTORY, HistoryTotal()))
      return (false);      

   if (CurTime() < ordersTime)
      return (false);
      
   if (CurTime() - ordersTime > (60 * 60))
      return (false);
      
   return (true);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+ 
bool OrdersFromCurrentSessionExists(datetime ordersTime, int mode, int totalOrders)
{
   for (int index =  totalOrders - 1; index >= 0; index--)
   {
      if (OrderSelect(index, SELECT_BY_POS, mode) == false)
         continue;
      
      if (OrderMagicNumber()!= MAGIC_NUMBER)
         continue;
         
      if (OrderSymbol() != pair.Symbol)
         continue;
      
      if (OrderOpenTime() >= ordersTime)
         return (true);
         
      if (CurTime() < ordersTime)
         return (true);
   }
   
   return (false);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+ 
void DetermineDailyRange(double& dailyRange[2])
{
   int rangeFirstBarIndex = FindRangeFirstBarIndex();
   if (rangeFirstBarIndex==0) 
      return;

   int highestHighBarIndex = Highest(pair.Symbol, chart.Period, MODE_HIGH, rangeFirstBarIndex, 1);
   double highestHighInRange = iHigh(pair.Symbol, chart.Period, highestHighBarIndex);

   int lowestLowBarIndex = Lowest(pair.Symbol, chart.Period, MODE_LOW, rangeFirstBarIndex, 1);
   double lowestLowInRange = iLow(pair.Symbol, chart.Period, lowestLowBarIndex);

   dailyRange[TOP_RANGE_BAND] = highestHighInRange + (Safty.Channel.Pips * pair.Point.Value) + normal.Spread.Value;
   dailyRange[BOTTOM_RANGE_BAND] = lowestLowInRange - (Safty.Channel.Pips * pair.Point.Value);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+ 
int FindRangeFirstBarIndex()
{
   datetime BarTime0 = iTime(pair.Symbol, chart.Period, 0);

   for (int index = 0; index < iBars(pair.Symbol, chart.Period); index++)
   {
      if (iTime(pair.Symbol, chart.Period, index) < StrToTime(TimeHour(BarTime0) + ":00") - (Range.Priod * 60 * 60)) 
         return (index - 1);
   }
   
   return (0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+ 
void HandleNewOrders(double dailyRange[2])
{
   int rangePips = MathRound((dailyRange[TOP_RANGE_BAND] - dailyRange[BOTTOM_RANGE_BAND]) / pair.Point.Value);
   if (rangePips > Max.Range.Pips)
      return;
         
   double Lots = Normal.Order.Size;
   
   double slPips = MathMin(Max.SL.Pips, rangePips);
   double slLong = dailyRange[TOP_RANGE_BAND] - (slPips * pair.Point.Value);
   double slShort = dailyRange[BOTTOM_RANGE_BAND] + (slPips * pair.Point.Value);

   double tpLong = dailyRange[TOP_RANGE_BAND] + (TP.Pips * pair.Point.Value);
   double tpShort = dailyRange[BOTTOM_RANGE_BAND] - (TP.Pips * pair.Point.Value);
   
   if (Normal.Order.Size == 0)
      Lots = LotsMM(slPips);

   double orderBuyPrice = dailyRange[TOP_RANGE_BAND];
   if (orderBuyPrice <= MarketInfo(pair.Symbol, MODE_ASK) + (pair.Stop.Level * pair.Point.Value))
      orderBuyPrice = MarketInfo(pair.Symbol, MODE_ASK) + ((pair.Stop.Level + 1) * pair.Point.Value);
   
   OrderSend(pair.Symbol, OP_BUYSTOP, Lots, orderBuyPrice, SLIPPAGE, slLong, tpLong, EA_NAME, MAGIC_NUMBER, TimeCurrent() + (60 * 60 * Pending.Trades.Expiration), Green);

   if (Normal.Order.Size == 0)
      Lots = LotsMM(slPips);

   double orderSellPrice = dailyRange[BOTTOM_RANGE_BAND];
   if (orderSellPrice >= MarketInfo(pair.Symbol, MODE_BID) - (pair.Stop.Level * pair.Point.Value))
      orderSellPrice = MarketInfo(pair.Symbol, MODE_BID) - ((pair.Stop.Level + 1) * pair.Point.Value);
      
   OrderSend(pair.Symbol, OP_SELLSTOP, Lots, orderSellPrice, SLIPPAGE, slShort, tpShort, EA_NAME, MAGIC_NUMBER, TimeCurrent() + (60 * 60 * Pending.Trades.Expiration), Red);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+ 
void HandleProfitLock()
{
   if (Lock.Profit.Trigger.Pips <= 0)
      return;

   int triggerPips = Lock.Profit.Trigger.Pips;
   int slTargetPips = Pips.To.Lock;
   int tpTargetPips = TP.Pips;
   double slValue, tpValue;
            
   for (int index = 0; index < OrdersTotal(); index++)
   {
      if (OrderSelect(index, SELECT_BY_POS, MODE_TRADES) == false)
         continue;
      
      if (OrderMagicNumber()!= MAGIC_NUMBER)
         continue;
         
      if (OrderSymbol() != pair.Symbol)
         continue;
      
      if (OrderType() == OP_BUY)
      {
         slValue = OrderOpenPrice() + slTargetPips * pair.Point.Value;
         tpValue = slValue + (tpTargetPips * pair.Point.Value);
         TrailCurrentBuyOrder(triggerPips, slValue, tpValue);
      }
      
      if (OrderType() == OP_SELL)
      {
         slValue = OrderOpenPrice() - slTargetPips * pair.Point.Value;
         tpValue = slValue - (tpTargetPips * pair.Point.Value);
         TrailCurrentSellOrder(triggerPips, slValue, tpValue);
      }
   }
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+ 
void HandleTrailingStop()
{
   if (Trailing.Stop.Trigger.Pips <= 0)
      return;
      
   int triggerPips = Trailing.Stop.Trigger.Pips;
   int slTargetPips = Pips.To.Lock;
   int tpTargetPips = TP.Pips;
   double slValue, tpValue;
            
   for (int index = 0; index < OrdersTotal(); index++)
   {
      if (OrderSelect(index, SELECT_BY_POS, MODE_TRADES) == false)
         continue;
      
      if (OrderMagicNumber()!= MAGIC_NUMBER)
         continue;
         
      if (OrderSymbol() != pair.Symbol)
         continue;
      
      if (OrderType() == OP_BUY)
      {
         slValue = MarketInfo(pair.Symbol, MODE_BID) - CalcTrailingSLValue();
         tpValue = slValue + (TP.Pips * pair.Point.Value);
         TrailCurrentBuyOrder(triggerPips, slValue, tpValue);
      }
      
      if (OrderType() == OP_SELL)
      {
         slValue = MarketInfo(pair.Symbol, MODE_ASK) + CalcTrailingSLValue();
         tpValue = slValue - (TP.Pips * pair.Point.Value);
         TrailCurrentSellOrder(triggerPips, slValue, tpValue);
      }
   }
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double CalcTrailingSLValue()
{
   double trailingSLValue = iATR(pair.Symbol, ATR_TRAILING_TIME_FRAME, ATR_TRAILING_PERIOD, ATR_TRAILING_BAR_SHIFT) * ATR.Trailing.SL.Factor;
      
   if (trailingSLValue < pair.Stop.Level * pair.Point.Value)
      trailingSLValue = pair.Stop.Level * pair.Point.Value;

   if (trailingSLValue > Max.Trailing.Pips * pair.Point.Value)
      trailingSLValue = Max.Trailing.Pips * pair.Point.Value;
            
   return (trailingSLValue);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool TrailCurrentBuyOrder(int triggerPips, double slValue, double tpValue)
{
   if (iHigh(pair.Symbol, chart.Period, 1) - OrderOpenPrice() < triggerPips * pair.Point.Value)
      return (false);

   if (slValue <= OrderStopLoss())
      return (false);
   
   return (OrderModify( OrderTicket(), 
                        OrderOpenPrice(), 
                        slValue,
                        tpValue,
                        0, 
                        Red));
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool TrailCurrentSellOrder(int triggerPips, double slValue, double tpValue)
{
   if (OrderOpenPrice() - (iLow(pair.Symbol, chart.Period, 1) + MarketInfo(pair.Symbol, MODE_SPREAD) * pair.Point.Value) < triggerPips * pair.Point.Value)
      return (false);
   
   if (slValue >= OrderStopLoss())
      return (false);
   
   return (OrderModify( OrderTicket(), 
                        OrderOpenPrice(), 
                        slValue,
                        tpValue,
                        0, 
                        Green));
}   

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+ 
double LotsMM(double slPips)
{
   double maxLoss4OneLot = slPips * pair.One.Pip.Value;
   
   double rv = NormalizeDouble((AccountFreeMargin() * Risk.Factor) / maxLoss4OneLot, 2);
   rv = NormalizeDouble(rv / pair.Lot.Step, 0) * pair.Lot.Step;
   
   if (rv < pair.Min.Lot) 
      return (pair.Min.Lot);
      
   if (rv > pair.Max.Lot) 
      return (pair.Max.Lot);

   return(rv);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+ 
double DrawLabels()
{
   string objName = "Copyright";
   int xPos = 8;
   int yPos = 15;
   int cornerID = 1;
   ObjectDelete(objName);
   ObjectCreate(objName, OBJ_LABEL, 0, 0, 0);
   ObjectSet(objName, OBJPROP_CORNER, cornerID);
   ObjectSet(objName, OBJPROP_XDISTANCE, xPos);
   ObjectSet(objName, OBJPROP_YDISTANCE, yPos);
   ObjectSet(objName, OBJPROP_BACK, true);
   ObjectSetText(objName, "www.TornadoFX.co.il", 10, "Arial", GreenYellow);
}