// Trend Manager.mq4
// Based on indicator sold at traderstradingsystem.com
// extended by www.easyforexsignals.com - check for more indicators on the site
// to include filters for solar wind, juice, adx


#property indicator_chart_window
#property indicator_buffers 4
#property indicator_color1 Red
#property indicator_color2 Blue
#property indicator_color3 Maroon
#property indicator_color4 Purple

#property indicator_width3 2;
#property indicator_width4 2;

extern int TM_Period = 7;
extern int TM_Shift = 2;
extern int MinADX = 20;
extern int MinRisingADX = 15;
extern double minOsma = 0.0002;
extern double minSolarWind = 0.2;
extern double minJPYOsma = 0.04;
extern double minJuice = 0.0001;
extern double minJPYJuice = 0.05;
extern int MoveArrowFromCandleInPoints = 25;
extern bool risingADXoverrides = true;


double SpanA_Buffer[];
double SpanB_Buffer[];

double BufferBuySignals[];
double BufferSellSignals[];

int a_begin;

int init()
{
   a_begin=TM_Shift;
   SetIndexStyle(0,DRAW_HISTOGRAM,STYLE_SOLID);
   SetIndexBuffer(0,SpanB_Buffer);
   SetIndexDrawBegin(0,TM_Period+a_begin-1);
   SetIndexLabel(0,"TM_Period+");
   SetIndexStyle(1,DRAW_HISTOGRAM,STYLE_SOLID);
   SetIndexBuffer(1,SpanA_Buffer);
   SetIndexDrawBegin(1,TM_Period+a_begin-1);
   SetIndexLabel(1,"TM_Period");
   
   SetIndexStyle(2,DRAW_ARROW);
   SetIndexBuffer(2,BufferBuySignals);
   SetIndexArrow(2,233); 
   SetIndexLabel(0,"BUY");
   
   SetIndexStyle(3,DRAW_ARROW);
   SetIndexBuffer(3,BufferSellSignals);
   SetIndexArrow(3,234); 
   SetIndexLabel(0,"SELL");

   IndicatorShortName("TRENDMGRFILTER");
   
   string SymbolText = Symbol();      
   if (StringLen(SymbolText) == 7){ 
      SymbolText = StringSubstr(SymbolText, 0,6);
   }
      
   if (SymbolText=="USDJPY") 
    {minOsma=minJPYOsma;
     minJuice=minJPYJuice;
     Print("Set values for USDJPY pair");
    
    }
   
   return(0);
}

int start()
{
   int    i,k;
   int    counted_bars=IndicatorCounted();
   double high,low,price;

   if(Bars<=TM_Period) return(0);
   if(counted_bars<1)
   {
      for(i=1;i<=TM_Period;i++) 
      {
         SpanA_Buffer[Bars-i]=0;
         SpanB_Buffer[Bars-i]=0;
         BufferBuySignals[Bars-i]=0;
         BufferSellSignals[Bars-i]=0;
      }
   }

   i=Bars-TM_Period;
   if(counted_bars>TM_Period) i=Bars-counted_bars-1;
   while(i>=0)
   {
      high=High[i]; low=Low[i]; k=i-1+TM_Period;
      while(k>=i)
      {
         price=High[k];
         if(high<price) high=price;
         price=Low[k];
         if(low>price)  low=price;
         k--;
      }
      SpanA_Buffer[i]  = (high+low)/2.0;
      SpanB_Buffer[i]  = SpanA_Buffer[i+TM_Shift];
      BufferBuySignals[i]=0;
      BufferSellSignals[i]=0;
      
      if (SpanA_Buffer[i] !=  SpanB_Buffer[i]) {
         if ((SpanA_Buffer[i] >  SpanB_Buffer[i]) && (SpanA_Buffer[i+1] <=  SpanB_Buffer[i+1])) {
            if (CheckAdx(i, OP_BUY)==true) {
               BufferBuySignals[i]=Low[i]-Point*MoveArrowFromCandleInPoints;
               if (i == 1) {
               //Alert("Buy on TM");
               }
            }
            else
            {
             BufferBuySignals[i]=0;
            }
         }
         else   if ((SpanA_Buffer[i] <  SpanB_Buffer[i]) && (SpanA_Buffer[i+1] >=  SpanB_Buffer[i+1])) {
            if (CheckAdx(i, OP_SELL)==true) {
            BufferSellSignals[i]=High[i]+Point*MoveArrowFromCandleInPoints;
              if (i == 1) {
              // Alert("Sell on TM");
               }
            }
            else{
            BufferSellSignals[i]=0;
            }
         }
      }
      i--;
   }

   return(0);
}

// check all filters here, not just adx
bool CheckAdx(int i, int OrderTyp) {
 bool retval = false;
   
 // get the WindowFind

 double Osma = iOsMA(Symbol(),Period(), 12,26,9,PRICE_CLOSE,i);
 double SolarWind = iCustom(Symbol(),Period(), "solar winds",0, i);
 double Juice = iCustom(Symbol(),Period(), "Juice",0, i);
 double redflattrend = iCustom(Symbol(),Period(), "flat trend",Period(),0, i);
 double greenflattrend = iCustom(Symbol(),Period(), "flat trend",Period(),1, i);
 //double yellowflattrend = iCustom(Symbol(),Period(), "flat trend",2, i);
 double adx = iADX(Symbol(),Period(), 14, PRICE_CLOSE, MODE_MAIN,i);
 double adxold = iADX(Symbol(),Period(), 14, PRICE_CLOSE, MODE_MAIN,i+1);
 bool adxvalid = false;
 
 
 if (risingADXoverrides ==true)
  {
      if (adxold < adx) // rising
         {
         if  (adx > MinRisingADX)
            {  adxvalid = true;}
         }
      else //falling
         {
         if  (adx > MinADX)
            {  adxvalid = true;}
         }
     
  }
  else
  {
   if (adx >= MinADX) {
      adxvalid = true;
   }   
  }
 
    if (OrderTyp == OP_BUY)
    {
      if ((Osma >= minOsma) && (SolarWind >= minSolarWind) && (Juice >= minJuice) && (adxvalid == true) && (greenflattrend>0) )
      //if ((Juice >= minJuice) && (adx >= MinADX)  )
         {
            retval = true;                     
         }
      else
         {
            retval = false;
         }
     }
     else if (OrderTyp == OP_SELL) 
     {
     if ((Osma <= -minOsma)  && (SolarWind <= minSolarWind) && (Juice >= minJuice) && (adxvalid == true) && (redflattrend>0)  )
      //   if ((Juice >= minJuice) && (adx >= MinADX)  )
         {
            retval = true;
         }
       else
         {
            retval = false;
         }
     }
  


     
   return(retval);
}